#include <windows.h>
#include <lsapi/lsapi.h>
#include "lsxcommand.h"
#include "modulelog.h"

/***********************************************************
* void ReadCommandChars()                                  *
*                         *  *  *  *                       *
*    Arguments:                                            *
*                                                          *
*    - struct CommandSettings *pSettings;                  *
*      A valid CommandSettings structure.                  *
*                         *  *  *  *                       *
* Reads the *CommandChars list.                            *
***********************************************************/
void ReadCommandChars(struct CommandSettings *pSettings)
{
  const unsigned nCommandChars = sizeof(pSettings->CommandChars)/sizeof(*pSettings->CommandChars);
  unsigned iChar;
  FILE *handle;
  char sLine[MAX_LINE_LENGTH];
  const char *pNextToken;

  for (iChar = 0; iChar < nCommandChars; ++iChar)
    pSettings->CommandChars[iChar] = NULL;

  handle = LCOpen(NULL);
  while (LCReadNextConfig(handle, "*CommandChar", sLine, MAX_LINE_LENGTH))
  {
    /* skip *CommandChar token */ 
    if (!GetToken(sLine, NULL, &pNextToken, FALSE)) continue;

    /* "*CommandChar stop" can be used to prevent themes */ 
    /* from unexpectedly defining command chars          */ 
    if (strnicmp(pNextToken, "stop", 4) == 0)
      break;

    if (!IS_COMMAND_CHAR(*pNextToken))
    {
      Complain("Invalid command char:%n%1", sLine);
      continue;
    }

    iChar = COMMAND_CHAR_INDEX(*pNextToken);
    if (pSettings->CommandChars[iChar] != NULL)
    {
      Complain("Duplicate definition for command char:%n%1%n%nAssigned command: %2",
        sLine, pSettings->CommandChars[iChar]);
      continue;
    }

    ++pNextToken;
    while (*pNextToken && *pNextToken <= ' ') ++pNextToken;

    if (*pNextToken == 0)
      continue;

    pSettings->CommandChars[iChar] = malloc(strlen(pNextToken) + 2);
    strcpy(pSettings->CommandChars[iChar], pNextToken);
    strcat(pSettings->CommandChars[iChar], " ");
  }
  LCClose(handle);

  /* set default for = to calculator */ 
  iChar = COMMAND_CHAR_INDEX('=');
  if (pSettings->CommandChars[iChar] == NULL)
  {
    static const char sExecCalculator[] = "!CommandCalculate =<args>";
    pSettings->CommandChars[iChar] = malloc(sizeof(sExecCalculator));
    strcpy(pSettings->CommandChars[iChar], sExecCalculator);
  }

  /* set default for ? to search query */ 
  iChar = COMMAND_CHAR_INDEX('?');
  if (pSettings->CommandChars[iChar] == NULL)
  {
    static const char sExecSearch[] = "!CommandSearch ?<args>";
    pSettings->CommandChars[iChar]= malloc(sizeof(sExecSearch));
    strcpy(pSettings->CommandChars[iChar], sExecSearch);
  }
}

/***********************************************************
* void FreeCommandChars()                                  *
*                         *  *  *  *                       *
*    Arguments:                                            *
*                                                          *
*    - struct CommandSettings *pSettings;                  *
*      A valid CommandSettings structure.                  *
*                         *  *  *  *                       *
* Frees the memory used by the command chars.              *
***********************************************************/
void FreeCommandChars(struct CommandSettings *pSettings)
{
  const unsigned nCommandChars = sizeof(pSettings->CommandChars)/sizeof(*pSettings->CommandChars);
  unsigned iChar;

  for (iChar = 0; iChar < nCommandChars; ++iChar)
  {
    free(pSettings->CommandChars[iChar]);
    pSettings->CommandChars[iChar] = NULL;
  }
}
