#ifndef MODULELOG_H_INCLUDED
#define MODULELOG_H_INCLUDED
#pragma once

#ifdef NDEBUG
#define NLOG
#endif

#ifndef NLOG

#ifndef _WINDOWS_
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#endif

#ifdef __cplusplus

/* C++ interface: create as many as you need, probably only one */ 

#include "str.h"

class ModuleLog
{
protected:
  const TCHAR *sFileName_;
  unsigned nIndent_;

public:
  ModuleLog(str sSettingPrefix, str sModuleName, str sModuleVersion);
  ~ModuleLog() { delete[] const_cast<TCHAR *>(sFileName_); }

  void WriteLine(str sLine, bool bTimeStamp = false);
  void Indent(int vDif) { nIndent_ += vDif; }

};

extern "C"
{
#endif /* __cplusplus */ 

/* C interface: only gets one instance of ModuleLog, probably sufficient */ 

void ModuleLogInit(const char *sSettingPrefix, unsigned nSettingPrefix,
                   const char *sModuleName, unsigned nModuleName,
                   const char *sModuleVersion, unsigned nModuleVersion);
void ModuleLogQuit(void);
void ModuleLogWriteLine(const char *sLine, unsigned nLine);
void ModuleLogWriteLineTimeStamp(const char *sLine, unsigned nLine);
void ModuleLogIndent(int vDif);

#ifdef __cplusplus
}
#endif /* __cplusplus */ 

#define TRACE1(STRING) ModuleLogWriteLineTimeStamp(TEXT(STRING), sizeof(TEXT(STRING))-1)

#else /* NLOG */ 

#define TRACE1(STRING) (void)0
#define ModuleLogInit(SSET,NSET,SNAME,NNAME,SVER,NVER) (void)0
#define ModuleLogQuit() (void)0

#endif /* NLOG */ 

#define STRINGIZE_EXPAND(TOKEN) STRINGIZE(TOKEN)
#define STRINGIZE(TOKEN) #TOKEN

#ifndef NLOG
#define REQUIRE(CONDITION) if (CONDITION) ; else TRACE1("FAIL: @" STRINGIZE_EXPAND(__LINE__) " REQUIRE(" #CONDITION ");")
#define ENSURE(CONDITION) if (CONDITION) ; else TRACE1("FAIL: @" STRINGIZE_EXPAND(__LINE__) " ENSURE(" #CONDITION ");")
#define ASSERT(CONDITION) if (CONDITION) ; else TRACE1("FAIL: @" STRINGIZE_EXPAND(__LINE__) " ASSERT(" #CONDITION ");")
#else
#define REQUIRE(CONDITION) (void)0
#define ENSURE(CONDITION) (void)0
#define ASSERT(CONDITION) (void)0
#endif

#define READABLEB(P, B) (!IsBadReadPtr(P, B))
#define READABLEN(P, N) READABLEB(P, (N)*sizeof(*(P)))
#define READABLE(P) READABLEN(P, 1)

#define WRITEABLEB(P, B) (!IsBadWritePtr(P, B))
#define WRITEABLEN(P, N) WRITEABLEB(P, (N)*sizeof(*(P)))
#define WRITEABLE(P) WRITEABLEN(P, 1)

#endif /* MODULELOG_H_INCLUDED */ 
